@@ -24,13 +24,11 @@ void lcmp (int *pop,
             double *ppos, 
             double *lpriorm, 
             int *burnintheta,
-	    double *lambdad,
-	    double *nud,
 	    int *verbose
 			 ) {
   int dimsample, Np;
   int step, staken, getone=1, intervalone=1, verboseMHcmp = 0;
-  int i, j, compute, ni, Ni, Ki, isamp, iinterval, isamplesize, iburnin;
+  int i, ni, Ni, Ki, isamp, iinterval, isamplesize, iburnin;
   double mui, sigmai, dsamp;
   double dkappa, ddf, dmu, dsigma, dmuproposal, dsigmaproposal;
   int tU, sizei, imaxN, imaxm, give_log0=0, give_log1=1;
@@ -176,69 +174,6 @@ void lcmp (int *pop,
     Ni += ni;
     if(Ni > imaxN) Ni = imaxN;
 		    
-    if((fabs(lambdad[0])>0.0000001) | (fabs(nud[0])>0.0000001)){
-Rprintf("No! lambdad[0] %f nud[0] %f\n", lambdad[0], nud[0]);
-    for (i=0; i<Ki; i++){
-      nk[i]=0;
-    }
-
-    /* Draw true degrees (sizes) based on the reported degrees*/
-    /* First find the reported degree distribution */
-    for (j=0; j<=maxpop; j++){
-//Rprintf("j %d pop[j] %d\n", j, pop[j]);
-     compute=0;
-     for (i=0; i<ni; i++){if(pop[i]==(j)){compute=1;}}
-     if(compute==1){
-//    Next four lines for cmp reporting distribution
-//    ?? Should it be cmp(j+1,...) or cmp(j,...)??
-      for (i=0; i<Ki; i++){
-       lzcmp = zcmp(exp(lambdad[i]),nud[i], errval, Ki, give_log1);
-//     pd[i]=pi[i]*cmp(j,lambdad[i],nud[i],lzcmp,give_log0);
-       pd[i]=pi[i]*cmp(j+1,lambdad[i],nud[i],lzcmp,give_log0);
-      }
-//     Next seven lines for proportional reporting distribution
-//       for (i=0; i<Ki; i++){
-//        pd[i]   = pgamma(2.0*lambdad[i]/((j)+0.5),1.0,1.0,0,0);
-//        if((j)>0){
-//         pd[i] -= pgamma(2.0*lambdad[i]/((j)-0.5),1.0,1.0,0,0);
-//        }
-//if((pd[i]<0.0 ) | (pd[i]>1.0)){ Rprintf("j %d pop[j] %d i %d pd[i] %f\n", j, pop[j],i, pd[i]);
-// Rprintf("i %d pi[i] %f, gammart %f\n", i, pi[i],  gammart);
-// }
-//       if(j==75 & isamp == 4){
-////      for (i=0; i<100; i++){
-//Rprintf("j %d dis %d i %d l[i] %f pd[i] %f\n", j, ddis, i, lambdad[i], pd[i]);
-//}// }
-//      pd[i]=p1i[i]*pd[i];
-//       }
-      // Set up pd to be cumulative for the random draws
-      for (i=1; i<Ki; i++){
-       pd[i]=pd[i-1]+pd[i];
-//if((pd[i]<0.0 ) | (pd[i]>1.0)){ Rprintf("j %d pop[j] %d i %d pd[i] %f\n", j, pop[j],i, pd[i]);}
-      }
-      /* Draw unobserved degrees sizes */
-      for (i=0; i<ni; i++){
-       if(pop[i]==(j)){
-        /* Now propose the true size for unit i based on reported size and disease status */
-        /* In the next three lines a sizei is chosen */
-        temp = pd[Ki-1] * unif_rand();
-        for (sizei=1; sizei<=Ki; sizei++){
-          if(temp <= pd[sizei-1]) break;
-        }
-        nk[sizei-1]=nk[sizei-1]+1;
-        d[i]=sizei;
-//Rprintf("j %d dis %d sizei %d pd[Ki-1] %f\n", j, ddis, sizei, pd[Ki-1]);
-       }
-      }
-     } //compute
-    } //for j
-    b[ni-1]=d[ni-1];
-    for (i=(ni-2); i>=0; i--){
-      b[i]=b[i+1]+d[i];
-    }
-// Rprintf("j %d d[j] %d pd[Ki-1] %f\n", j, d[j], pd[Ki-1]);
-    }
-
     /* Draw phis */
     tU=0;
     for (i=ni; i<Ni; i++){
