@@ -24,17 +24,15 @@ void gcmpdisease (int *pop, int *dis,
             double *ppos, 
 	    double *lpriorm,
             int *burnintheta,
-	    double *lambdad,
-	    double *nud,
             int *verbose
               ) {
   int dimsample, Np0, Np1;
   int step, staken, getone=1, intervalone=1, verboseMHdisease = 0;
-  int i, j, compute, ni, Ni, Ki, isamp, iinterval, isamplesize, iburnin;
+  int i, ni, Ni, Ki, isamp, iinterval, isamplesize, iburnin;
   double mu0i, mu1i, pbeta, beta, sigma0i, sigma1i, dsamp;
   double dkappa0, ddf0, dmu0, dmu1, dsigma0, dsigma1, dmuproposal, dsigmaproposal;
   int tU, sizei, imaxN, imaxm, itotdis0, itotdis, give_log0=0, give_log1=1;
-  int maxpop, ddis;
+  int maxpop;
   double r, gamma0rt, gamma1rt, p0is, p1is, Nd;
   double gammart, temp;
   double errval=0.000000001, lzcmp;
@@ -241,95 +239,6 @@ void gcmpdisease (int *pop, int *dis,
 //  if (*verbose) Rprintf("step %d Ni %d itotdis %d beta %f mu0 %f mu1 %f s0 %f r %f\n",
 //  step, Ni, itotdis, betasample[0], musample[0], musample[1], sigmasample[0], r);
 
-// Rprintf("lambdad[0] %f\n", lambdad[0]);
-// Rprintf("nud[0] %f\n", nud[0]);
-    if((fabs(lambdad[0])>0.0000001) | (fabs(nud[0])>0.0000001)){
-// Rprintf("lambdad[0] %f\n", lambdad[0]);
-    for (i=0; i<Ki; i++){
-      nk0[i]=0;
-      nk1[i]=0;
-    }
-
-    /* Draw true degrees (sizes) based on the reported degrees*/
-    /* First find the reported degree distribution */
-    for (j=0; j<=maxpop; j++){
-    for (ddis=0; ddis<2; ddis++){
-//Rprintf("j %d pop[j] %d\n", j, pop[j]);
-     compute=0;
-     for (i=0; i<ni; i++){if((pop[i]==(j)) && (dis[i]==ddis)){compute=1;}}
-     if(compute==1){
-      if(ddis==1){
-//    Next four lines for cmp reporting distribution
-//    ?? Should it be cmp(j+1,...) or cmp(j,...)??
-//    I think j=1 as pd[i], p1i[i], etc, refer to degree i+1 
-//    as all degrees must be positive.
-      for (i=0; i<Ki; i++){
-       lzcmp = zcmp(exp(lambdad[i]),nud[i], errval, 2*Ki, give_log1);
-       pd[i]=p1i[i]*cmp(j+1,lambdad[i],nud[i],lzcmp,give_log0);
-      }
-//     Next seven lines for proportional reporting distribution
-//       for (i=0; i<Ki; i++){
-//        pd[i]   = pgamma(2.0*lambdad[i]/((j)+0.5),1.0,1.0,0,0);
-//        if((j)>0){
-//         pd[i] -= pgamma(2.0*lambdad[i]/((j)-0.5),1.0,1.0,0,0);
-//        }
-// if((pd[i]<0.0 ) | (pd[i]>1.0)){ Rprintf("j %d pop[j] %d i %d pd[i] %f\n", j, pop[j],i, pd[i]);
-//  Rprintf("i %d p1i[i] %f, gamma0rt %f gamma1rt %f \n", i, p1i[i],  gamma0rt,  gamma1rt);
-//  }
-//       if(j==75 & isamp == 4){
-////      for (i=0; i<100; i++){
-//Rprintf("j %d dis %d i %d l[i] %f pd[i] %f\n", j, ddis, i, lambdad[i], pd[i]);
-//}// }
-//      pd[i]=p1i[i]*pd[i];
-//       }
-      }else{
-//      Next four lines for cmp reporting distribution
-       for (i=0; i<Ki; i++){
-        lzcmp = zcmp(exp(lambdad[i]),nud[i], errval, 2*Ki, give_log1);
-        pd[i]=p0i[i]*cmp(j,lambdad[i],nud[i],lzcmp,give_log0);
-       }
-//     Next seven lines for proportional reporting distribution
-//       for (i=0; i<Ki; i++){
-//        pd[i]   = pgamma(2.0*lambdad[i]/((j)+0.5),1.0,1.0,0,0);
-//        if((j)>0){
-//         pd[i] -= pgamma(2.0*lambdad[i]/((j)-0.5),1.0,1.0,0,0);
-//        }
-//      pd[i]=p0i[i]*pd[i];
-// Rprintf("i %d pd[i] %f, p0i[i] %f, pop[i] %d lambdad[i] %f nud[i] %f \n", i, log(pd[i]), p0i[i], pop[i], lambdad[i],nud[i]);
-//       }
-      }
-      // Set up pd to be cumulative for the random draws
-      for (i=1; i<Ki; i++){
-       pd[i]=pd[i-1]+pd[i];
-// if((pd[i]<0.0 ) | (pd[i]>1.0)){ Rprintf("j %d pop[j] %d i %d pd[i] %f\n", j, pop[j],i, pd[i]);}
-      }
-      /* Draw unobserved degrees sizes */
-      for (i=0; i<ni; i++){
-       if((pop[i]==(j)) && (dis[i]==ddis)){
-        /* Now propose the true size for unit i based on reported size and disease status */
-        /* In the next three lines a sizei is chosen */
-        temp = pd[Ki-1] * unif_rand();
-        for (sizei=1; sizei<=Ki; sizei++){
-          if(temp <= pd[sizei-1]) break;
-        }
-        if(dis[i]==1){
-          nk1[sizei-1]=nk1[sizei-1]+1;
-        }else{
-          nk0[sizei-1]=nk0[sizei-1]+1;
-        }
-        d[i]=sizei;
-//Rprintf("j %d dis %d sizei %d pd[Ki-1] %f\n", j, ddis, sizei, pd[Ki-1]);
-       }
-      }
-     } //compute
-    }} //for j and ddis
-    b[ni-1]=d[ni-1];
-    for (i=(ni-2); i>=0; i--){
-      b[i]=b[i+1]+d[i];
-    }
-// Rprintf("j %d d[j] %d pd[Ki-1] %f\n", j, d[j], pd[Ki-1]);
-    }
-
     /* Draw phis */
     tU=0;
     for (i=ni; i<Ni; i++){
